#include "gameengine.h"
#include "gamestate.h"

#include "rgl.h"
#include "soundsystem.h"

#include "inc.h"

extern SoundSystem sound;

GameEngine::GameEngine() {
	this->states = NULL;
}
GameEngine::~GameEngine() {
	this->states = NULL;
}

void GameEngine::init(const char* title, int width, int height, int bpp, bool fullscreen)
{
	printf("\n:. GameEngine init..\n");

	int flags = 0;

	// Init SDL
	printf(" Initializing SDL .. ");
	setup_sdl("Traction::Game", width, height, bpp, fullscreen);
	printf("done.\n");

	// Init Opengl
	printf(" Initializing OpenGL .. ");
	setup_opengl(width, height);
	printf("done.\n");

	// Init soundsystem
	printf(" Initializing soundsystem ..\n");
	sound.init();		
	printf(" done.\n");

	// Load textures
	printf(" Loading textures.. \n");
	RLoadTextures(tex);
	printf(" done.\n");

	// Build fonts
	printf(" Building fonts.. ");
	BuildFont(tex);
	printf("done.\n");

	this->clock = Timer();

	fullscreen = fullscreen;
	run = true;
	this->bpp = bpp;
	this->screen_height = width;
	this->screen_height = height;
	this->fullscreen = fullscreen;

	printf(":. GameEngine init finished\n\n");
}

void GameEngine::free()
{	
	printf("\n:. GameEngine Cleanup..\n");

	// cleanup the state
	if( states!=NULL ) {
		states->free();
	}

	printf(" Releasing fonts.. ");
	FreeFont(tex);
	printf("done\n");
	
	printf(" Releasing textures.. ");
	RFreeTextures(tex);
	printf("done\n");


	/*
	// switch back to windowed mode so other 
	// programs won't get accidentally resized
	// is this needed in new sdl ? can't recall if it got fixed or not (?!)
	if ( m_fullscreen ) {
		screen = SDL_SetVideoMode(640, 480, 0, 0);
	}*/

	printf(":. GameEngine cleanup finished\n\n");
		
	// shutdown SDL
	SDL_Quit();
}

void GameEngine::changeState(GameState* state) 
{
	
	printf(": Moving to state %s\n", state->getStateName().c_str());

	// cleanup the current state
	if ( this->states!=NULL ) 
    {
        printf("state is not null");
        printf("state = %x\n", states);
        fflush(stdout);
		states->free();
    }

    printf("initing new state");
    fflush(stdout);

	// store and init the new state
	this->states = state;
	this->states->init();
}

// This is the basis of the whole thing
void GameEngine::handleEvents() 
{
	// let the state handle events
	states->handleEvents(this);
}

void GameEngine::update() 
{

	this->clock.fpsthink();

	// let the state update the game
	states->update(this);
}

void GameEngine::draw() 
{
	// let the state draw the screen
	states->draw(this);
}

