//////////////////////////////////////////////////////////////////////
// mapstate.cpp: implementation of the playstate class.
//////////////////////////////////////////////////////////////////////

#include "mapstate.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MapState::MapState()
{
    printf("MapState::MapState()\n");
    printf("address = %x\n", this);
	this->statename = "MapState";

    fflush(stdout);
}

MapState::~MapState()
{
    printf("Mapstate::~MapState()\n");
}

const int TILE_EMPTY = 0;
const int TILE_FIELD = 1;
const int TILE_CITY  = 2;
const int MAX_TILE_TYPE_COUNT = 3;

const int UNIT_PRODUCE_TIME = 5;
const int UNIT_MAX_COUNT = 3;

const int TURN_PLACE_TILE = 0;
const int TURN_MOVE_UNITS = 1;
const int TURN_EVENTS_DONE = 2;

const char *tile_names[MAX_TILE_TYPE_COUNT] = 
{
    "Empty (shouldn't happen)\0",
    "Field\0",
    "City\0",
};

void MapState::init() 
{
    int x, y;

    printf("Mapstate::init()\n");

    gridResX = 30;
    gridResY = 30;

    gridSizeY = 1.0f;
    gridSizeX = 1.0f;

    selectedTileX = 0;
    selectedTileY = 0;
    selectedUnit = 0;

    gameTurns = 0;

    grid = new MapTile[gridResX*gridResY];

    //loopataan ja initoidaan gridi
    for (y=0;y<gridResY;y++)
    {
        for (x=0;x<gridResX;x++)
        {
            grid[x+y*gridResX].active = false;
            grid[x+y*gridResX].visible = false;
            grid[x+y*gridResX].produces = false;
            grid[x+y*gridResX].type = TILE_EMPTY;
            grid[x+y*gridResX].x = x;
            grid[x+y*gridResY].y = y;
            grid[x+y*gridResY].worldPos = Vector(x-gridResX*0.5f, y-gridResY*0.5f, 0);
        }
    }

    initNextTile();
    units = new Lista();

    beginTurn();
}

void MapState::beginTurn()
{
    printf("*************************** MapState::beginTurn()\n");
    printf("alkanut vuoro = %d\n", gameTurns);
    gameTurnPosition = 0;
    currentUnit = 0;
    selectedUnit = 0;
}

void MapState::endTurn()
{
    printf("*************************** MapState::endTurn()\n");
    printf("loppunut vuoro = %d\n", gameTurns);
    processTiles();
    gameTurns++;
}

void MapState::drawTile(int tileX, int tileY)
{
    if ((tileX < 0) || (tileY < 0) || (tileX > gridResX) || (tileY > gridResY))
    {
        printf("MapState::drawTile() : Cannot draw tile at (%d, %d)", tileX, tileY);
        return;
    }

    MapTile *tile = &grid[tileX+tileY*gridResX];

    Vector v1 = tile->worldPos;//Vector(t->x, 0, t->y);//t->worldPos;
    Vector v2 = v1 + Vector(gridSizeX, 0, 0);
    Vector v3 = v1 + Vector(gridSizeX, gridSizeY, 0);
    Vector v4 = v1 + Vector(0, gridSizeY, 0);

    float r = 0.0;
    float g = 0.0;
    float b = 0.0;

    switch(tile->type)
    {
        case TILE_EMPTY:
        {
            r = 0.2f;
            g = 0.2f;
            b = 0.3f;
        } break;

        case TILE_FIELD:
        {
            r = 0.2f;
            g = 0.5f;
            b = 0.2f;
        } break;

        case TILE_CITY:
        {
            r = 0.5f;
            g = 0.5f;
            b = 0.5f;
        } break;

        default:
        {
            r = 1.0f;
            g = 0.0f;
            b = 0.0f;
        }
    }

    glColor4f(r, g, b, 1);



    glBegin(GL_QUADS);
    glVertex3fv((float *)&v1);
    glVertex3fv((float *)&v2);
    glVertex3fv((float *)&v3);
    glVertex3fv((float *)&v4);
    glEnd();
/*
    if (grid[tileX+tileY*gridResX].active)
    {
        glColor4ub(255, 255, 255, 255);
        glBegin(GL_LINE_LOOP);
        glVertex3fv((float *)&v1);
        glVertex3fv((float *)&v2);
        glVertex3fv((float *)&v3);
        glVertex3fv((float *)&v4);
        glEnd();
    }
*/
}
void MapState::drawSelectedTile()
{
    if ((selectedTileX < 0) || (selectedTileY < 0) || (selectedTileX > gridResX) || (selectedTileY > gridResY))
    {
        printf("MapState::drawSelectedTile() : Cannot draw tile at (%d, %d)", selectedTileX, selectedTileY);
        return;
    }
    if (gameTurnPosition == TURN_PLACE_TILE)
    {
        Vector v1 = grid[selectedTileX+selectedTileY*gridResX].worldPos;//Vector(t->x, 0, t->y);//t->worldPos;
        Vector v2 = v1 + Vector(gridSizeX, 0, 0);
        Vector v3 = v1 + Vector(gridSizeX, gridSizeY, 0);
        Vector v4 = v1 + Vector(0, gridSizeY, 0);

        glLineWidth(2);
        glEnable(GL_LINE_SMOOTH);
        glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);       
        glColor4f(1, 0.0f, 0.0f, 0.7f+0.2f*(float)sin(animationValue));
    
        glBegin(GL_LINE_LOOP);
        glVertex3fv((float *)&v1);
        glVertex3fv((float *)&v2);
        glVertex3fv((float *)&v3);
        glVertex3fv((float *)&v4);
        glEnd();
        glDisable(GL_BLEND);

        glLineWidth(1);
    }
}

void MapState::draw(GameEngine *game) 
{
    static float distangle = 0.0f;

//    distangle += game->clock.deltatime;

    glLoadIdentity();
    glTranslatef(0, 0, -30+1*(float)sin(distangle*0.001f));
    glRotatef(180, 1, 0, 0);
/*
    static int framecount = 0;
    glRotatef(framecount*0.01f, 0, 0, 0);
    framecount++;
*/
    //piirr tilet
    drawTiles();
    //piirr unitit
    drawUnits();


    glEnable(GL_TEXTURE_2D);
    glColor4f(1,1,1,1);

    char buf[256]; 
	RPrint(game->tex, 10, 30, "MapState", 0);
    sprintf(buf, "selected = %d,%d\0", selectedTileX, selectedTileY);
	RPrint(game->tex, 10, 45, buf, 0);

    sprintf(buf, "next tile = %s\0", tile_names[nextTileType]);
    RPrint(game->tex, 10, 60, buf, 0);

    sprintf(buf, "game turn = %d\0", gameTurns);
    RPrint(game->tex, 10, 75, buf, 0);

    switch(gameTurnPosition)
    {
        case TURN_PLACE_TILE: sprintf(buf, "Place a tile"); break;
        case TURN_MOVE_UNITS: sprintf(buf, "Move your units"); break;
        default: sprintf(buf, "Rukoile, koska joku kusee nyt"); break;
    }
    RPrint(game->tex, 10, 90, buf, 0);

}

void MapState::drawTiles()
{
    int x, y;
    glDisable(GL_BLEND);
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_DEPTH_TEST);

    for (y=0;y<gridResY;y++)
    {
        for (x=0;x<gridResX;x++)
        {
            drawTile(x, y);
        }
    }

    drawSelectedTile();
}

void MapState::drawUnits()
{
    int i;
    int unitcount = units->getNodeCount();
    if (unitcount > 0)
    {
        units->goToStart();

        glDisable(GL_TEXTURE_2D);
        glDisable(GL_DEPTH_TEST);
        glColor4f(1,1,1,1);
        float size = 0.5f*gridSizeX;
        for (i=0;i<unitcount;i++)
        {
            MapUnit *unit = (MapUnit *)units->getCurrent ();

            Vector worldPos = grid[unit->x + unit->y*gridResX].worldPos;
            Vector v1 = worldPos;
            Vector v2 = v1 + Vector(size, 0, 0);
            Vector v3 = v1 + Vector(size, size, 0);
            Vector v4 = v1 + Vector(0, size, 0);

            glBegin(GL_QUADS);
            glVertex3fv((float *)&v1);
            glVertex3fv((float *)&v2);
            glVertex3fv((float *)&v3);
            glVertex3fv((float *)&v4);

            glEnd();
            units->goForward();
        }

        if (gameTurnPosition == TURN_MOVE_UNITS)
        {
            units->goToNode(currentUnit);
            MapUnit *selected = (MapUnit *)units->getCurrent();
            glEnable(GL_BLEND);
            glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);       
            glColor4f(1, 0, 0, 0.7f+0.2f*(float)sin(animationValue));

            Vector v1 = grid[selected->x + selected->y*gridResX].worldPos;
            Vector v2 = v1 + Vector(size, 0, 0);
            Vector v3 = v1 + Vector(size, size, 0);
            Vector v4 = v1 + Vector(0, size, 0);

            glLineWidth(2);
            glBegin(GL_LINE_LOOP);
            glVertex3fv((float *)&v1);
            glVertex3fv((float *)&v2);
            glVertex3fv((float *)&v3);
            glVertex3fv((float *)&v4);
            glEnd();
            glDisable(GL_BLEND);
            glLineWidth(1);
        }
    }
}

void MapState::update(GameEngine *game) 
{ 
    animationTimer += game->clock.deltatime;
    animationValue = animationTimer * 0.01f;
}


void MapState::processTiles()
{
    printf("MapState::processTiles()\n");
    int x, y;
    for (y=0;y<gridResY;y++)
    {
        for (x=0;x<gridResX;x++)
        {
            MapTile *tile = &grid[x+y*gridResX];

            if (tile->produces)
            {
                tile->timeToProduction--;
                if (tile->timeToProduction==0)
                {
                    int unitcount = units->getNodeCount();
                    if (unitcount < UNIT_MAX_COUNT)
                    {
                        MapUnit *unit = new MapUnit();

                        unit->x = tile->x;
                        unit->y = tile->y;
                        unit->type = 0;

                        units->add(unit);
                        printf("MapState::processTiles() : unit produced at (%d, %d)\n", unit->x, unit->y);
                    }
                    tile->timeToProduction = UNIT_PRODUCE_TIME;
                }
            }
        }
    }
}


void MapState::activateTile(int tileX, int tileY)
{
    if ((tileX < 0) || (tileY < 0) || (tileX > gridResX) || (tileY > gridResY))
    {
        printf("MapState::activateTile() : Cannot activate tile at (%d, %d)", tileX, tileY);
        return;
    }
    if (!grid[tileX+tileY*gridResX].active)
    {
        grid[tileX+tileY*gridResX].active = true;
        printf("MapState::activateTile() : activated %d, %d\n", tileX, tileY);
        return;
    }
}
bool MapState::setTile(int tileX, int tileY)
{
    if ((tileX < 0) || (tileY < 0) || (tileX > gridResX) || (tileY > gridResY))
    {
        return false;
    }
    if (grid[tileX+tileY*gridResX].active)
    {
        return false;
    }
    else
    {
        int type = nextTileType;
        MapTile *tile = &grid[tileX+tileY*gridResX];
        tile->type = type;
        tile->active = true;

        switch(tile->type)
        {
            case TILE_CITY:
            {
                tile->produces = true;
                tile->timeToProduction = UNIT_PRODUCE_TIME;
            } break;

            default:
            {
            }
        }
        initNextTile();
        return true;
    }
}

void MapState::initNextTile()
{
    int randvalue = 1 + (rand()%(MAX_TILE_TYPE_COUNT-1));
    nextTileType = randvalue;
}

bool MapState::clipCoordinates(int *tileX, int *tileY)
{
    bool status = true;
    if (*tileX > gridResX-1)
    {
        *tileX = gridResX-1;
        status = false;
    }
    if (*tileX < 0)
    {
        *tileX = 0;
        status = false;
    }
    if (*tileY > gridResY-1)
    {
        *tileY = gridResY-1;
        status = false;
    }
    if (*tileY < 0)
    {
        *tileY = 0;
        status = false;
    }
    return status;

}
void MapState::handleEvents(GameEngine *game) 
{
    bool actionDone = false;
    bool movementDone = false;
    int unitmovement = 0;

    const int UNIT_MOVE_UP = 1;
    const int UNIT_MOVE_DOWN = 2;
    const int UNIT_MOVE_LEFT = 3;
    const int UNIT_MOVE_RIGHT = 4;

	while ( SDL_PollEvent(&game->event) ) 
    {
	switch (game->event.type) 
    {
		case SDL_QUIT:
			game->quit();
		break;

        case SDL_MOUSEBUTTONDOWN:
            {

                printf("naks naks\n");
            }

            break;

        
        case SDL_KEYDOWN:
            switch( game->event.key.keysym.sym )
            {
				case SDLK_UP:
                    {
                        if (gameTurnPosition == TURN_PLACE_TILE)
                        {
                            selectedTileY--;
                        }
                        if (gameTurnPosition == TURN_MOVE_UNITS)
                        {
                            movementDone = true;
                            unitmovement = UNIT_MOVE_UP;
                        }
                    } 
                    break;
                case SDLK_DOWN:
                    {
                        if (gameTurnPosition == TURN_PLACE_TILE)
                        {
                            selectedTileY++;
                        }
                        if (gameTurnPosition == TURN_MOVE_UNITS)
                        {
                            movementDone = true;
                            unitmovement = UNIT_MOVE_DOWN;
                        }
                    }
					break;
				case SDLK_LEFT:
                    {
                        if (gameTurnPosition == TURN_PLACE_TILE)
                        {
                            selectedTileX--;
                        }
                        if (gameTurnPosition == TURN_MOVE_UNITS)
                        {
                            movementDone = true;
                            unitmovement = UNIT_MOVE_LEFT;
                        }

                    }
					break;
				case SDLK_RIGHT:
                    {
                        if (gameTurnPosition == TURN_PLACE_TILE)
                        {
                            selectedTileX++;
                        }
                        if (gameTurnPosition == TURN_MOVE_UNITS)
                        {
                            movementDone = true;
                            unitmovement = UNIT_MOVE_RIGHT;
                        }

                    }
					break;
                case SDLK_SPACE:
                    {
                        actionDone = true;
                    }
                    break;
				case SDLK_ESCAPE:
					game->quit();
					break;
				default:
                    break;
			  }
	       break;
        case SDL_KEYUP:
            switch( game->event.key.keysym.sym ){
				case SDLK_RCTRL:	
				case SDLK_UP:
					break;
                case SDLK_DOWN:
					break;
                case SDLK_LEFT:
					break;
				case SDLK_RIGHT:
                    break;
                case SDLK_RETURN:
					break;
				default:
                    break;
			  }
	       break;
		default:
			break;			
		}
	}

    clipCoordinates(&selectedTileX, &selectedTileY);

    if (actionDone)
    {
        if (gameTurnPosition==TURN_PLACE_TILE)
        {
            if (setTile(selectedTileX, selectedTileY))
            {
                gameTurnPosition = TURN_MOVE_UNITS;
                currentUnit = 0;
            }
            else
            {
            }
        }
    }
    if (movementDone)
    {
        units->goToNode(currentUnit);
        MapUnit *unit = (MapUnit *)units->getCurrent();

        int x2 = unit->x;
        int y2 = unit->y;
        switch(unitmovement)
        {
            case UNIT_MOVE_UP: y2--; break;
            case UNIT_MOVE_DOWN: y2++; break;
            case UNIT_MOVE_LEFT: x2--; break;
            case UNIT_MOVE_RIGHT: x2++; break;
        }

        if (clipCoordinates(&x2, &y2))
        {
            unit->x = x2;
            unit->y = y2;
            currentUnit++;
        }
    }

    //jos tytyy liikuttaa unitteja
    if (gameTurnPosition == TURN_MOVE_UNITS)
    {
        int unitcount = units->getNodeCount();
        //jos ei ole unitteja tai kaikki on jo siirretty
        if ((unitcount <= 0) || (currentUnit==unitcount))
        {
            gameTurnPosition = TURN_EVENTS_DONE;
        }
        else
        {

        }
    }
    
    if (gameTurnPosition == TURN_EVENTS_DONE)
    {
        endTurn();
        beginTurn();
    }
}
void MapState::pause() 
{
}
void MapState::resume() 
{
}
void MapState::free() 
{
    printf("MapState::free()\n");
    delete [] grid;
    delete units;
}

