#include "rgl.h"
#include <stdlib.h>
#include <string.h>

void RLoadTextures(Textures &tex) {
  tex.font	 = RLoadTextures("./data/font.png");
  tex.layer1 = RLoadTextures("./data/tiles.png");
  tex.isochars = RLoadTextures("./data/isocharacters.png");
}

void RFreeTextures(Textures &tex) {
	
	glDeleteTextures(1,&tex.font);
	glDeleteTextures(1,&tex.layer1);
	
}

void RPreRender() {

		// Clear background buffers with black. 
		glClearColor(0.0f, 0.0f, 0.0f, 0.0f);	
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);			
			
		// Do not remove 'invisible' surfaces 
		glDisable( GL_DEPTH_TEST );
		
		// Reset translations, rotations and scalings
		glMatrixMode( GL_MODELVIEW ) ;		
		glLoadIdentity() ;

}

// FIX: 14.6 - Piirt defaulttina z=-50 tasolle
// FIX: 13.6 - piirsi quadien tekstuurit mirroroidusti
// Toimii 512 x 512 tekstuureille
bool RBlit(RRect to, GLuint textureid, RRect from, GLfloat rotation, GLfloat scale, GLfloat z) {

	glColor3f(1,1,1);

	glDisable(GL_DEPTH_TEST);			

	glPushMatrix();

		// Siirrot
		int move_left	= to.getW()/2;
		int move_top	= to.getH()/2;

		glScalef(scale,scale,scale);
		glTranslatef((float)to.getX()+move_left, (float)to.getY()+move_top, z);
		//	glTranslatef(-1*(float)to.getY(),-1*(float)to.getX(),0);
		glRotatef(rotation,0,0,1);

		// Texture coordinates as float	
		float from_x, from_y, from_x2, from_y2;

		from_x  = from.getX()/512.0f;
		from_y  = from.getY()/512.0f;
		from_x2 = (from.getX()+from.getW())/512.0f;
		from_y2 = (from.getY()+from.getH())/512.0f;

        // tell opengl to use the texture
        glBindTexture(GL_TEXTURE_2D, textureid);
        glEnable(GL_TEXTURE_2D);

        // make a rectangle
        glBegin(GL_QUADS);

         // top left
         glTexCoord2f(from_x2, from_y); 
		 glVertex3i(move_left, move_top, 0);

         // top right
         glTexCoord2f(from_x, from_y);
         glVertex3i(-move_left,move_top, 0);

         // bottom right
         glTexCoord2f(from_x, from_y2); 
         glVertex3i(-move_left,-move_top, 0);

         // bottom left
		 glTexCoord2f(from_x2, from_y2);
         glVertex3i(move_left, -move_top, 0);

        glEnd();

	glPopMatrix();
	
	glEnable(GL_DEPTH_TEST);			

	return true;
}

// Loads texture from file and splits it into textures
GLuint RLoadTextures(char *filename) {

        SDL_Surface *surface;
        GLuint textureid;
        int mode;

		surface = IMG_Load(filename);

        // could not load filename
        if (!surface) {
				printf("  Failed to load %s\n", filename);
                return 0;
        }

        // work out what format to tell glTexImage2D to use...
        if (surface->format->BytesPerPixel == 3) { // RGB 24bit
                mode = GL_RGB;
        } else if (surface->format->BytesPerPixel == 4) { // RGBA 32bit
                mode = GL_RGBA;
        } else {
                SDL_FreeSurface(surface);
                return 0;
        }

        // create one texture name
        glGenTextures(1, &textureid);

        // tell opengl to use the generated texture name
        glBindTexture(GL_TEXTURE_2D, textureid);

        // this reads from the sdl surface and puts it into an opengl texture (with mipmaps)
		gluBuild2DMipmaps(GL_TEXTURE_2D, 4, surface->w, surface->h, mode, GL_UNSIGNED_BYTE, surface->pixels);

		// if no mipmaps, use this instead:	
		//glTexImage2D(GL_TEXTURE_2D, 0, mode, surface->w, surface->h, 0, mode, GL_UNSIGNED_BYTE, surface->pixels);

        // these affect how this texture is drawn later on...
        //glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
        // glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);	

        // clean up
        SDL_FreeSurface(surface);

        return textureid;

}

// bitmap font routines  - from NEHE lesson#17
//  could have written own, but this display list version seems worth of 
//  learning :)
void BuildFont(Textures &tx)							// Build Our Font Display List
{
	float	cx;											// Holds Our X Character Coord
	float	cy;											// Holds Our Y Character Coord
	int loop;

	tx.font_list=glGenLists(256);						// Creating 256 Display Lists
	glBindTexture(GL_TEXTURE_2D, tx.font);				// Select Our Font Texture
	for (loop=0; loop<256; loop++)						// Loop Through All 256 Lists
	{
		cx=float(loop%16)/16.0f;						// X Position Of Current Character
		cy=float(loop/16)/16.0f;						// Y Position Of Current Character

		glNewList(tx.font_list+loop, GL_COMPILE);		// Start Building A List
			glBegin(GL_QUADS);							// Use A Quad For Each Character

				glTexCoord2f(cx,1-cy-0.0625f);			// Texture Coord (Bottom Left)
				glVertex2i(0,0);						// Vertex Coord (Bottom Left)
				glTexCoord2f(cx+0.0625f,1-cy-0.0625f);	// Texture Coord (Bottom Right)
				glVertex2i(16,0);						// Vertex Coord (Bottom Right)
				glTexCoord2f(cx+0.0625f,1-cy);			// Texture Coord (Top Right)
				glVertex2i(16,16);						// Vertex Coord (Top Right)
				glTexCoord2f(cx,1-cy);					// Texture Coord (Top Left)
				glVertex2i(0,16);						// Vertex Coord (Top Left)

			glEnd();									// Done Building Our Quad (Character)
			glTranslated(10,0,0);						// Move To The Right Of The Character
		glEndList();									// Done Building The Display List
	}													// Loop Until All 256 Are Built
}

void FreeFont(Textures &tx)									// Delete The Font From Memory
{
	glDeleteLists(tx.font_list ,256);							// Delete All 256 Display Lists
}

// Yksinkertaisuuden vuoksi kytt aina 800 x 600 screeni - helppo vaihtaa resoa..
void RPrint(Textures &tx, GLint x, GLint y, char *string, int set)	// Where The Printing Happens
{
	// siirretn fontti kohalleen
	y = 600-y;

	if (set>1)	{   set=0;	}
	glBindTexture(GL_TEXTURE_2D, tx.font);				// Select Our Font Texture
	
	glDisable(GL_DEPTH_TEST);							// Disables Depth Testing
	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPushMatrix();										// Store The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix
	glOrtho(0,800,0,600,-1,1);							// Set Up An Ortho Screen

	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glPushMatrix();										// Store The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix 
	glTranslated(x,y,0);								// Position The Text (0,0 - Bottom Left)

	 glListBase(tx.font_list-32+(128*set));					// Choose The Font Set (0 or 1)
	 glCallLists(strlen(string), GL_UNSIGNED_BYTE, string);	// Write The Text To The Screen

	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPopMatrix();										// Restore The Old Projection Matrix
	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glPopMatrix();										// Restore The Old Projection Matrix
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
}


void setup_sdl(const char* title, int width, int height, int bpp, bool fullscreen)
{
    const SDL_VideoInfo* video;

    if ( SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO) < 0 ) {
        fprintf(stderr,
                "Couldn't initialize SDL: %s\n", SDL_GetError());
        exit(1);
    }

    atexit(SDL_Quit);

    video = SDL_GetVideoInfo( );

    if( !video ) {
        fprintf(stderr,
                "Couldn't get video information: %s\n", SDL_GetError());
        exit(1);
    }

    // Set the minimum requirements for the OpenGL window 
    SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
				
	// Asetetaan ikoni ja topicbar viesti 
	SDL_WM_SetCaption(title, "icon");
	
	int flags = 0;
	flags = SDL_OPENGL;
	if(fullscreen) 
		flags = SDL_OPENGL | SDL_FULLSCREEN;

	 // SDL_GL_DOUBLEBUFFER attribuuttia ei tarvita opengl:n kanssa     
	if( SDL_SetVideoMode( width, height, video->vfmt->BitsPerPixel, flags) == 0 ) {
		fprintf(stderr, 
				"Couldn't set video mode: %s\n", SDL_GetError());
		exit(1);
	}

}


void setup_opengl(int width, int height)
{
    float aspect = (float)width/ (float)height;

    glViewport(0, 0, width, height);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    
    gluPerspective(60.0f, aspect, 1.0f, 15000.0f);
	

    glMatrixMode(GL_MODELVIEW);

    glClearColor(1.0f, 1.0f, 1.0f, 0);
    
    // Enable Z-Buffer
    glEnable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D);

    // Draw polygons even if normals are in wrong direction
    glDisable(GL_CULL_FACE);

	// Alpha channel for transparency
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

}

// dunno if this is needed - but here it is :)
void setup_sdl_joystick() {

			SDL_JoystickEventState(SDL_ENABLE);
			SDL_Joystick *joy;
			// Check for joystick
			if(SDL_NumJoysticks()>0){
			
			// Open joystick0, hope they don't have more than one :)
			joy=SDL_JoystickOpen(0);

				if(joy)
				{
					
					printf("Opened Joystick 0\n");
					printf("Name: %s\n", SDL_JoystickName(0));
					printf("Number of Axes: %d\n", SDL_JoystickNumAxes(joy));
					printf("Number of Buttons: %d\n", SDL_JoystickNumButtons(joy));
					printf("Number of Balls: %d\n", SDL_JoystickNumBalls(joy));
				}
				else
					printf("Couldn't open Joystick 0\n");
				  
			} else printf("No joysticks available\n");
}


// ------------------------------------------------------------
// Timer
// ------------------------------------------------------------
Timer::Timer() {


		// Set all frame times to 0ms.
        memset(frametimes, 0, sizeof(frametimes));	// FIXME: We should check if we have memory here!
        framecount = 0;
        framespersecond = 0;
        frametimelast = SDL_GetTicks();
		secsperframe = 0;
		deltatime = 0;
}
Timer::~Timer() { }

void Timer::fpsthink() {

        Uint32 frametimesindex;
        Uint32 getticks;
        Uint32 count;
        Uint32 i;

        // frametimesindex is the position in the array. It ranges from 0 to FRAME_VALUES.
        // This value rotates back to 0 after it hits FRAME_VALUES.
        frametimesindex = framecount % FRAME_VALUES;

        // store the current time
        getticks = SDL_GetTicks();
		
		// Make game run at about MAX_FPS frames per second
		if(getticks-frametimelast<1000/MAX_FPS) 
			  SDL_Delay( Uint32 ((1000/MAX_FPS) - (getticks-frametimelast)) );

		getticks = SDL_GetTicks();
		deltatime = getticks - frametimelast;		// calculate time since last update

        // save the frame time value
        frametimes[frametimesindex] = getticks - frametimelast;
		
		// Calculate the time between frames
		secsperframe = (getticks - frametimelast)/1000.0f;
		
        // save the last frame time for the next fpsthink
        frametimelast = getticks;
		
        // increment the frame count
        framecount++;

        // Work out the current framerate
        // The code below could be moved into another function if you don't need the value every frame.
        if (framecount < FRAME_VALUES) {
                count = framecount;
        } else {
                count = FRAME_VALUES;
        }

        // add up all the values and divide to get the average frame time.
        framespersecond = 0;
        for (i = 0; i < count; i++) {
                framespersecond += frametimes[i];
        }

        framespersecond /= count;

        // now to make it an actual frames per second value...
        framespersecond = 1000.f / framespersecond;

}


// Palauttaa hiiren osumapisteen planella
// Parametrit:
//  x = mouse_position_x
//  y = mouse_position_y
//  z = planen z koordinaatti jolta osumaa haetaan (plane on x ja y akselien suuntainen)
// EX: Vector location = getMouseCoord(game->event.motion.x, game->event.motion.y, 0);
Vector getMouseCoord(int x, int y, int z)
{
	// #define _DEBUG

	Vector raypoint1, raypoint2, direction;

	GLint viewport[4];
	GLdouble modelview[16];
	GLdouble projection[16];
	GLfloat winX, winY, winZ;
	GLdouble posX, posY, posZ;

	// Fetch needed projection modifiers
	glGetDoublev( GL_MODELVIEW_MATRIX, modelview );
	glGetDoublev( GL_PROJECTION_MATRIX, projection );
	glGetIntegerv( GL_VIEWPORT, viewport );

	winX = (float)x;
	winY = (float)viewport[3] - (float)y;

	// Fetch projected mouse position (on nearplane)
	winZ=0.0;
	gluUnProject( winX, winY, winZ, modelview, projection, viewport, &posX, &posY, &posZ);
	raypoint1 = Vector((float)posX, (float)posY, (float)posZ);

	// Fetch projected mouse position (on farplane)
	winZ=1.0;
	gluUnProject( winX, winY, winZ, modelview, projection, viewport, &posX, &posY, &posZ);
	raypoint2 = Vector((float)posX, (float)posY, (float)posZ);

	// Calculate the direction of the ray
	direction = raypoint2-raypoint1;
	direction = direction.normalize();

	// Find the multiplier for vector equation
	float tt;
	if (direction.z != 0)
		if(direction.z<0)
			tt = (	raypoint2.z - z) / direction.z;
		else 
			tt = (	raypoint2.z + z) / direction.z;
	else
		tt = 0.0f;

	// Switch -1 => 1 and vice versa
	tt*=-1;	

	// Calculate the point of intersection and return it
	Vector intersect = raypoint2  + direction*tt;
	
	#ifndef _DEBUG	
		printf("Mouseray/plane intersection: %f, %f, %f z\n", intersect.x, intersect.y, intersect.z);
	#endif

	return intersect;
}

