#ifndef _RGL_H_
#define _RGL_H_

#include <stdlib.h> // for exit(1)
#ifndef LINUX
#include "SDL.h"
#include "SDL_image.h"
#include "SDL_opengl.h"
#else
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#endif
#include "inc.h"


#include "vector.h"

class RRect {

	public:
		// Constructor
		RRect() {
			this->x=0;
			this->y=0;
			this->w=512;
			this->h=512;
		}
		RRect(int gx,int gy,int gw,int gh) { 
			this->x=gx;
			this->y=gy;
			this->w=gw;
			this->h=gh;
		}
		int setX(int gx) { this->x=gx; }
		int setY(int gy) { this->y=gy; }
		int setW(int gw) { this->w=gw; }
		int setH(int gh) { this->h=gh; }

		int getX() { return this->x; }
		int getY() { return this->y; }
		int getW() { return this->w; }
		int getH() { return this->h; }

	private:
		int x, y, w, h;
};

// Texture identifiers are stored here
struct Textures {
	GLuint font;
	GLuint font_list;
	GLuint isochars;
	GLuint layer1;
	GLuint layer2;
};

bool RBlit(RRect to, GLuint textureid, RRect from, GLfloat rotation=0.0f, GLfloat scale=1.0f, GLfloat z=-50.0f);
GLuint RLoadTextures(char *filename);
void RFreeTextures(Textures &tex);
void RLoadTextures(Textures &tex);
void RPreRender();

// Font routines
void BuildFont(Textures &tx);
void FreeFont(Textures &tx);
void RPrint(Textures &tx, int x, int y, char *string, int set);

void setup_opengl(int width, int height);
void setup_sdl(const char* title, int width, int height, int bpp, bool fullscreen);
void setup_sdl_joystick();


Vector getMouseCoord(int x, int y, int z);

// how many frame values we want to store
const int FRAME_VALUES = 10;


class Timer {

	public: 
		
		Timer();
		~Timer();

		Uint32 frametimes[FRAME_VALUES];	// An array to store frame times:
		Uint32 frametimelast;				// Last calculated SDL_GetTicks
		Uint32 framecount;					// total frames rendered
		float framespersecond;				// Frames per second
		float secsperframe;
		Uint32 deltatime;				// milliseconds

		void fpsthink();		// advance timer, must be called each tick.
};


Vector getMouseCoord(int x, int y);
Vector getMouseCoord(int x, int y, int z);

#endif
