#include "soundsystem.h"


SoundSystem::SoundSystem(void)
{
	name_e[0] = "fx1.wav";
	name_e[1] = "fx2.wav";

	name_m[0] = "dubvibes.ogg";

	this->loading_error = false;

}

SoundSystem::~SoundSystem(void)
{
}

void SoundSystem::init() {

		// Format we want	
		int audio_rate = 44100;
		Uint16 audio_format = AUDIO_S16;	// byteorder
		int audio_channels = 2;
		int audio_buffers = 4096;

		// Init sdl_mixer (22KHz,signed 16bit, system byte order, stereo audio, using 1024 byte chunks
		//if(Mix_OpenAudio(44100, AUDIO_S16, 2, 1024)==-1) {
		if(Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_buffers)==-1) {
			printf("  Mix_OpenAudio: %s\n", Mix_GetError());
		} else {
			Mix_QuerySpec(&audio_rate, &audio_format, &audio_channels);
			printf("  Mixer opened with specs:\n   Frequency: %i\n   Channels: %i\n",audio_rate, audio_channels);
		}


		// Lets get 16 channels for audio
		Mix_AllocateChannels(16);


		string path = "./data/";
		string filename = "";

		// Load effects
		for(int i=0; i<NUM_EFF; i++) {
			
			filename = path + name_e[i];
			effect[i]=Mix_LoadWAV(filename.c_str());
			if(!effect[i]) {
				printf("  Mix_LoadWAV: %s\n", Mix_GetError());
				this->loading_error = true;
			} else printf("  Loaded: %s\n",filename.c_str());

		}

		// Load musics
		for(int i=0; i<NUM_MUS; i++) {
			
			filename = path + name_m[i];

			music[i]=Mix_LoadMUS(filename.c_str());
			if(!music[i]) {
				printf(" Mix_LoadMUS: %s\n", Mix_GetError());
				this->loading_error = true;
			} else printf("  Loaded: %s\n",filename.c_str());
		}

}

void SoundSystem::close() {
		
		// Free memory reserved for musics
		for(int i=0; i<NUM_MUS; i++)
			Mix_FreeMusic(this->music[i]);

		// Free memory reserved for effects
		for(int i=0; i<NUM_EFF; i++)
			Mix_FreeChunk(this->effect[i]);

		// Free allocated FX channels
		Mix_AllocateChannels(0);

		// Close down SDl_mixer
		Mix_CloseAudio();
}

// Play a music till its stopped (Stops previous music!)
// i = music num to play 
void SoundSystem::playMusic(int i) {
	
	if(this->loading_error==false) {

		// If legal music num, play it
		if(i>=0 && i<NUM_MUS)
		 
		 // musicfile, times to play (-1 forever), fadein time (ms)
		 if(Mix_FadeInMusic(this->music[i], -1, 5000)==-1) {
			 printf("Mix_PlayMusic: %s\n", Mix_GetError());
		 }

	}

}


// Stops music
// time = to fade the music out (ms) (if -1) then fade 2000ms
void SoundSystem::stopMusic(int time) {
	if(time<0)
		time=1500;
		Mix_FadeOutMusic(time);
    
}

// Plays soundfx
void SoundSystem::play(int i) {

	if(this->loading_error==false) {

		if(i>=0 && i<NUM_EFF)
		 if(Mix_PlayChannel(-1, effect[i], 0)==-1) {
			printf("Mix_PlayChannel: %s\n",Mix_GetError());	
		 }
	}
}
